#include "algoritmosGrafos.h"
#include <iostream>
   using namespace std;
namespace algoritmosGrafos {

   /**
    * Devuelve un conjunto de aristas que forma un rbol de recubrimiento
    * de coste mnimo del grafo g
    * @return un array con las aristas del rbol de recubrimiento de 
    * coste mnimo
    */
   Arista *arbolDeRecubrimiento(GrafoCostes g) {
   
      Arista *aristas = new Arista[g.vertices() - 1];
      bool *conectados = new bool[g.vertices()];
      CosteMin *costesBajos = new CosteMin[g.vertices()];
   
      // Se toma el vrtice 0 como primer vrtice del rbol de 
      // recubrimiento y se calculan las aristas de coste mnimo
      // desde el vrtice 0 a cada uno de los otros
      for (int i = 0; i < g.vertices(); i++) {
         conectados[i] = false;
         costesBajos[i].vertice = 0;
         costesBajos[i].coste = g.coste(0, i);
      }
      conectados[0] = true;
   
      // Se completa el rbol de recubrimiento seleccionando en
      // cada iteracin la arista de coste mnimo entre los vrtices
      // conectados y los no conectados.
      for (int i = 1; i < g.vertices(); i++) {
         int minimo = INT_MAX;
         int vertMin = -1;
         for (int j = 1; j < g.vertices(); j++) {
            if ((!conectados[j]) && (costesBajos[j].coste) < minimo) {
               minimo = costesBajos[j].coste;
               vertMin = j;
            }
         }
      
         // Se incluye la arista seleccionada en el rbol de
         // recubrimiento de coste mnimo
         aristas[i - 1].v1 = vertMin;
         aristas[i - 1].v2 =  costesBajos[vertMin].vertice;
         aristas[i - 1].coste = costesBajos[vertMin].coste;
         conectados[vertMin]  = true;
      
         // Se recalculan las aristas de coste mnimo entre los vrtices
         // conectados y los no conectados teniendo en cuenta el vrtice
         // recin conectado
         for (int j = 1; j < g.vertices(); j++) {
            if ((!conectados[j]) && (g.coste(vertMin, j) < costesBajos[j].coste)) {
               costesBajos[j].vertice = vertMin;
               costesBajos[j].coste = g.coste(vertMin, j);
            }
         }	
      }
      delete conectados;
      delete costesBajos;
      return aristas;
   }


   /**
    * Calcula el coste y vrtice responsable de cada camino
    * de coste mnimo deste fuente a cada vrtice de g
    * @return un array con los costes de los caminos de la fuente a cada 
    * vrtice 
    */
   CosteMin *costesDesdeFuente(GrafoCostes g, int fuente) {
      CosteMin *caminos = new CosteMin[g.vertices()];
      bool *tratados = new bool[g.vertices()];
   
      // Inicializar el conjunto de vrtices tratados con la fuente y 
      // calcular el coste directo desde la fuente a cada vrtice
      for (int i = 0; i < g.vertices(); i++) {
         tratados[i] = false;
         caminos[i].vertice = (g.conectados(fuente, i)) ? fuente : -1;
         caminos[i].coste = g.coste(fuente, i);
      }
   
      // A la fuente se le asigna un coste directo cero 
      caminos[fuente].vertice = fuente;
      caminos[fuente].coste = 0;
      tratados[fuente] = true;
   
      // Se calculan los costes mnimos desde fuente a 
      // todos los dems vrtices
      for (int j = 0; j < g.vertices() - 1; j++) {
         //Se determina el nuevo vrtice pendiente de coste
         //mnimo desde Fuente
         int costeMinimo, costeAux, vertCosteMin;
         costeMinimo = INT_MAX;
         vertCosteMin = -1;
      
         for (int i = 0; i < g.vertices(); i++) {
            if (!tratados[i])
               if (caminos[i].coste <= costeMinimo) {
                  costeMinimo = caminos[i].coste;
                  vertCosteMin = i;
               }
         }
      
         // Se aade el nuevo vrtice de coste mnimo al conjunto de
         // los tratados y se actualizan los costes de los caminos en
         // funcin del nuevo vrtice
         tratados[vertCosteMin] = true;
         for (int i = 0; i < g.vertices(); i++) {
            if (!tratados[i]) {
               if ((INT_MAX - costeMinimo) > g.coste(vertCosteMin, i))
                  costeAux = costeMinimo + g.coste(vertCosteMin, i);
               else
                  costeAux = INT_MAX;
            
               if (caminos[i].coste > costeAux) {
                  caminos[i].coste = costeAux;
                  caminos[i].vertice = vertCosteMin;
               }
            }
         }
      }
      delete tratados;
      return caminos;
   }

   /**
    * Recorre un grafo en amplitud
    */ 
   void amplitud(Grafo g, void (*tratar)(int)) {   
      // enCurso se usa como una cola capaz de mantener
      // todos los vrtices de G
      queue<int> enCurso;
      bool *visitados = new bool[g.vertices()];
      int v; // vrtice actual
   
      // Se inicializa visitados, que se pondr a true a medida que 
      // se visiten los vrtices.
      for (int i = 0; i < g.vertices(); i++)
         visitados[i] = false;
   
      // El recorrido en amplitud se inicia en cada vrtice
      // an no visitado.
      for (int i = 0; i < g.vertices(); i++)
         if (!visitados[i]) {
            // Se pone en la cola el vrtice de partida
            // y se marca como visitado.
            enCurso.push(i);
            visitados[i] = true;
         
            while (!enCurso.empty()) {
               // Se toma el vrtice que est en el frente
               // de la cola y se trata.
               v = enCurso.front();
               enCurso.pop();
               tratar(v);
            
               // Todos los vrtices adyacentes a v no visitados
               // se incorporan a la cola y se marcan.
               for (int j = 0; j < g.vertices(); j++) {
                  if ((v != j)
                     && (g.conectados(v,j) && (!visitados[j]))) {
                     enCurso.push(j);
                     visitados[j] = true;
                  }
               }
            }
         }
   }

   /**
    * Recorre en profundidad los vrtices de g accesibles desde v,
    * tratndolos y marcndolos como visitados.
    */           
   void avanzarEnProfundidad(
                     Grafo &g,
                     int v,
                     bool *visitados, 
                     void (*tratar)(int)) {
   
      // El vrtice v se marca como visitado	
      visitados[v] = true;
   
      //Se trata el vrtice
      tratar(v);
   
      // Se examinan los vrtices adyacentes a v para
      // continuar el recorrido.
      for (int i = 0; i < g.vertices(); i++)
         if ((v != i) && (!visitados[i]) && (g.conectados(v, i)))
            avanzarEnProfundidad(g, i, visitados, tratar);
   }

   /**
    * Recorre un grafo en profundidad
    */
   void profundidad(Grafo &g, void (*tratar)(int)) {
      bool *visitados = new bool[g.vertices()];
   
      // Se inicializa visitados, que se pondr a true a medida que 
      // se visiten los vrtices.
      for (int i = 0; i < g.vertices(); i++)
         visitados[i] = false;
   
      // El recorrido se relanza en cada vrtice no visitado
      // hasta que lo estn todos.
      for (int i = 0; i < g.vertices(); i++)
         if (!visitados[i])
            avanzarEnProfundidad(g, i, visitados, tratar);
   }


}
