package grafos;
import java.util.*;

/**
 * Algoritmos de tratamiento de grafos
 */
public abstract class AlgoritmosGrafos {
	// Clase interna auxiliar, representa un vrtice y un coste relacionado con l
	public static class CosteMin {
		int vrtice, coste;

		public CosteMin() {
			super();
		}
		
		public CosteMin(int v, int c) {
			vrtice = v;
			coste = c;
		}
	}

	// Clase interna auxiliar, representa una arista y su coste
	public static class Arista {
		int v1, v2, coste;

		public Arista() {
			super();
		}
		
		public Arista(int a, int b, int c) {
			v1 = a;
			v2 = b;
			coste = c;
		}
	}
	
	public void tratar(int i) {};

	/**
	 * Devuelve un conjunto de aristas que forma un rbol de recubrimiento
	 * de coste mnimo del grafo g
	 * @return un array con las aristas del rbol de recubrimiento de coste 
	 *         mnimo
	 */
	public static Arista[] rbolDeRecubrimiento(GrafoCostes g) {
		Arista[] aristas = new Arista[g.vrtices() - 1];
		boolean[] conectados = new boolean[g.vrtices()];
		CosteMin[] costesBajos = new CosteMin[g.vrtices()];

		// Se toma el vrtice 0 como primer vrtice del rbol de 
		// recubrimiento y se calculan las aristas de coste mnimo
		// desde el vrtice 0 a cada uno de los otros
		for (int i = 0; i < conectados.length; i++) {
			conectados[i] = false;
			costesBajos[i] = new CosteMin(0, g.coste(0, i));
		}
		conectados[0] = true;

		// Se completa el rbol de recubrimiento seleccionando en
		// cada iteracin la arista de coste mnimo entre los vrtices
		// conectados y los no conectados.
		for (int i = 1; i < g.vrtices(); i++) {
			int mnimo = Integer.MAX_VALUE;
			int vertMin = -1;
			for (int j = 2; j < g.vrtices(); j++) {
				if ((!conectados[j]) && (costesBajos[j].coste) < mnimo) {
					mnimo = costesBajos[j].coste;
					vertMin = j;
				}
			}

			// Se incluye la arista seleccionada en el rbol de
			// recubrimiento de coste mnimo
			aristas[i - 1] = new Arista();
			aristas[i - 1].v1 = vertMin;
			aristas[i - 1].v2 =  costesBajos[vertMin].vrtice;
			aristas[i - 1].coste = costesBajos[vertMin].coste;
			conectados[vertMin]  = true;
			
			// Se recalculan las aristas de coste mnimo entre los vrtices
			// conectados y los no conectados teniendo en cuenta el vrtice
			// recin conectado
			for (int j = 1; j < g.vrtices(); j++) {
				if ((!conectados[j]) && (g.coste(vertMin, j) < costesBajos[j].coste)) {
					costesBajos[j] = new CosteMin(vertMin, g.coste(vertMin, j));
				}
			}	
		}

		return aristas;
	}

	/**
	 * Calcula el coste y vrtice responsable de cada camino de coste mnimo
	 * deste fuente a cada vrtice de g
	 * @return un array con los costes de los caminos de la fuente a cada
	 *         vrtice
	 */
	public static CosteMin[] costesDesdeFuente(GrafoCostes g, int fuente) {
		CosteMin[] caminos = new CosteMin[g.vrtices()];
		boolean[] tratados = new boolean[g.vrtices()];

		// Inicializar el conjunto de vrtices tratados con la fuente y
		// calcular el coste directo desde la fuente a cada vrtice
		for (int i = 0; i < g.vrtices(); i++) {
			tratados[i] = false;
			caminos[i] = new CosteMin();
			caminos[i].vrtice = (g.conectados(fuente, i)) ? fuente : -1;
			caminos[i].coste = g.coste(fuente, i);
		}

		// A la fuente se le asigna un coste directo cero 
		caminos[fuente].vrtice = fuente;
		caminos[fuente].coste = 0;
		tratados[fuente] = true;

		// Se calculan los costes mnimos desde fuente a 
		// todos los dems vrtices
		for (int j = 0; j < g.vrtices() - 1; j++) {
			//Se determina el nuevo vrtice pendiente de coste
			//mnimo desde Fuente
			int costeMnimo, costeAux, vertCosteMin;
			costeMnimo = Integer.MAX_VALUE;
			vertCosteMin = -1;

			for (int i = 0; i < g.vrtices(); i++) {
				if (!tratados[i])
					if (caminos[i].coste <= costeMnimo) {
						costeMnimo = caminos[i].coste;
						vertCosteMin = i;
					}
			}

			// Se aade el nuevo vrtice de coste mnimo al conjunto de
			// los tratados y se actualizan los costes de los caminos en
			// funcin del nuevo vrtice
			tratados[vertCosteMin] = true;
			for (int i = 0; i < g.vrtices(); i++) {
				if (!tratados[i]) {
					if ((Integer.MAX_VALUE - costeMnimo) > g.coste(vertCosteMin, i))
						costeAux = costeMnimo + g.coste(vertCosteMin, i);
					else
						costeAux = Integer.MAX_VALUE;

					if (caminos[i].coste > costeAux) {
						caminos[i].coste = costeAux;
						caminos[i].vrtice = vertCosteMin;
					}
				}
			}
		}

		return caminos;
	}

	/**
	 * Recorre en profundidad los vrtices de g accesibles desde v,
	 * tratndolos y marcndolos como visitados.
	 */
	private void avanzarEnProfundidad(
		Grafo g,
		int v,
		boolean[] visitados) {

		// El vrtice v se marca como visitado	
		visitados[v] = true;

		//Se trata el vrtice
		tratar(v);

		// Se examinan los vrtices adyacentes a v para
		// continuar el recorrido.
		for (int i = 0; i < g.vrtices(); i++)
			if ((v != i) && (!visitados[i]) && (g.conectados(v, i)))
				avanzarEnProfundidad(g, i, visitados);
	}

	/**
	 * Recorre un grafo en profundidad
	 * @param g, grafo a recorrer
	 */
	public void profundidad(Grafo g) {
		boolean visitados[] = new boolean[g.vrtices()];

		// Se inicializa visitados, que se pondr a true a medida que 
		// se visiten los vrtices.
		for (int i = 0; i < g.vrtices(); i++)
			visitados[i] = false;

		// El recorrido se relanza en cada vrtice no visitado
		// hasta que lo estn todos.
		for (int i = 0; i < g.vrtices(); i++)
			if (!visitados[i])
				avanzarEnProfundidad(g, i, visitados);
	}

	/**
	 * Recorre un grafo en amplitud
	 * @param g, grafo a recorrer
	 */
	public void amplitud(Grafo g) {
		// Ls lista enCurso se usa como una cola capaz de mantener
		// todos los vrtices de G
		LinkedList enCurso = new LinkedList();
		boolean visitados[] = new boolean[g.vrtices()];
		int v; // vrtice actual

		// Se inicializa visitados, que se pondr a true a medida que 
		// se visiten los vrtices.
		for (int i = 0; i < g.vrtices(); i++)
			visitados[i] = false;

		// El recorrido en amplitud se inicia en cada vrtice
		// an no visitado.
		for (int i = 0; i < g.vrtices(); i++)
			if (!visitados[i]) {
				// Se pone en la cola el vrtice de partida
				// y se marca como visitado.
				enCurso.addLast(new Integer(i));
				visitados[i] = true;

				while (!enCurso.isEmpty()) {
					// Se toma el vrtice que est en el frente
					// de la cola y se trata.
					v = ((Integer) enCurso.removeFirst()).intValue();
					// tratar
					tratar(v);

					// Todos los vrtices adyacentes a v no visitados
					// se incorporran a la cola y se marcan.
					for (int j = 0; j < g.vrtices(); j++) {
						if ((v != j)
							&& (g.conectados(v, j) && (!visitados[j]))) {
							enCurso.addLast(new Integer(j));
							visitados[j] = true;
						}
					}
				}
			}
	}

}
